SystemOrganization addCategory: #'CS528-UT3Server'!

Object subclass: #UT3MockWebSocket
	instanceVariableNames: 'msgBlock isDead message sentMessages lastMessage defaultDelayMs'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-UT3Server'!

!UT3MockWebSocket commentStamp: '<historical>' prior: 0!
An UT3MockWebSocket is a mocked web socket for testing.

Instance Variables
!

!UT3MockWebSocket methodsFor: 'initialize-release' stamp: 'ShineLow 7/20/2013 11:01'!
initialize
	defaultDelayMs := 10.
	sentMessages := ''.
	message := nil.
	isDead := false.! !

!UT3MockWebSocket methodsFor: 'starting/stopping' stamp: 'ShineLow 7/20/2013 11:01'!
kill
	isDead := true.
	"Wait until some time for system to be processed."
	( Delay forMilliseconds: defaultDelayMs ) wait.! !

!UT3MockWebSocket methodsFor: 'communication' stamp: 'ShineLow 7/20/2013 11:02'!
receiveMessage: aString
	[ message isNil ] whileFalse: [ ( Delay forMilliseconds: defaultDelayMs ) wait ].
	message := aString.
	[ message isNil ] whileTrue: [ ( Delay forMilliseconds: defaultDelayMs ) wait ].
	"Wait until some time for system to be processed."
	( Delay forMilliseconds: defaultDelayMs ) wait.! !

!UT3MockWebSocket methodsFor: 'communication' stamp: 'ShineLow 7/20/2013 11:01'!
receiveMessageAndExpect: aString
	[ message isNil ] whileFalse: [ ( Delay forMilliseconds: defaultDelayMs ) wait ].
	lastMessage := nil.
	message := aString.
	[ message isNil ] whileTrue: [ ( Delay forMilliseconds: defaultDelayMs ) wait ].
	"Wait until some time for system to be processed."
	( Delay forMilliseconds: defaultDelayMs ) wait.
	[lastMessage isNil] whileTrue: [ ( Delay forMilliseconds: defaultDelayMs ) wait ].
	^ lastMessage! !

!UT3MockWebSocket methodsFor: 'protocol' stamp: 'ShineLow 7/20/2013 06:13'!
runWith: aBlock
	msgBlock := aBlock.
	[ isDead not ]
		whileTrue: [ message isNil ifFalse: [ msgBlock value: message. message := nil ].
					( Delay forMilliseconds: 1 ) wait ].
	ConnectionClosed signal: 'Mock socket killed'.
	isDead := false.! !

!UT3MockWebSocket methodsFor: 'communication' stamp: 'ShineLow 7/20/2013 09:17'!
sendMessage: aString
	sentMessages := sentMessages,Character cr asString,aString.
	lastMessage := aString! !

WAComponent subclass: #UT3WebServer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-UT3Server'!

!UT3WebServer commentStamp: '<historical>' prior: 0!
An UT3WebServer is a component that holds everything to do with the web server to play the game.

Instance Variables
!

UT3WebServer subclass: #UT3GameSession
	instanceVariableNames: 'system row column subrow subcolumn ssubrow ssubcolumn withRandomAi withEasyAi withMediumAi withOutAi withHardAi withInsaneAi withPerfectAi'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-UT3Server'!

!UT3GameSession commentStamp: '<historical>' prior: 0!
An UT3GameSession is a web component holding a session of a game (ie a System).

Instance Variables
	system:		<Object>

system
	- xxxxx
!

!UT3GameSession methodsFor: 'accessing' stamp: 'ShineLow 7/4/2013 20:38'!
column
	^ column asString! !

!UT3GameSession methodsFor: 'accessing' stamp: 'ShineLow 7/4/2013 20:39'!
column: aColumn
	column := aColumn asInteger! !

!UT3GameSession methodsFor: 'initialization' stamp: 'Oliver 7/25/2013 16:43'!
initialize
	super initialize.
	system := UT3System new.
	row := 1.
	column := 1.
	subrow := 1.
	subcolumn := 1.
	withRandomAi := false.
	withEasyAi := false.
	withMediumAi := false.
	withHardAi := false.
	withInsaneAi := false.
	withOutAi := true.
	ssubrow := 1.
	ssubcolumn := 1! !

!UT3GameSession methodsFor: 'initialization' stamp: 'Oliver 7/25/2013 16:45'!
makeMove
	system currentGameType = #basic
	ifTrue: [ system player: system currentPlayer moveRow: row column: column ]
	ifFalse: [ 
		( (system currentGameType = #medium) | (system currentGameType = #large) |  (system currentGameType = #huge) |  (system currentGameType = #gigantic) |  (system currentGameType = #enormous)  |  (system currentGameType = #gomoku) )
		ifTrue: [ system player: system currentPlayer moveRow: row column: column ]
		ifFalse: [ 
			system currentGameType = #ultimate
			ifTrue: [ system player: system currentPlayer moveAt: { row. column } subBoard: { subrow. subcolumn }  ]
			ifFalse: [ 
				system currentGameType = #double_ultimate
				ifTrue: [ system player: system currentPlayer moveAt: { row. column } subBoard: { subrow. subcolumn } ssubBoard: { ssubrow. ssubcolumn } 
					]	
				]
			]
		]! !

!UT3GameSession methodsFor: 'initialization' stamp: 'Oliver 7/25/2013 16:26'!
renderBoardOn: html
	| board subboard ssubboard cell |
	board := system board.
	" render the board"
	((system currentGameType = #basic) | (system currentGameType = #medium) | (system currentGameType = #large) |
	(system currentGameType = #huge) | (system currentGameType = #gigantic) | (system currentGameType = #enormous) | (system currentGameType = #gomoku))
	ifTrue: [ 
		board indicesDo: [ :aRow :aCol | 
			cell := board at: aRow at: aCol.
			((cell isEmpty) and: ((board winner) = #undecided))
			ifTrue:	[ html anchor callback: [system player: system currentPlayer moveAt: {aRow . aCol} ]; with: (cell printString) ]
			ifFalse: [ html text: (cell printString) ].
			html text: '|'.
			aCol = board size
			ifTrue: [ html break.] 
		] 
	].
	
	(system currentGameType = #connect4) 
	ifTrue: [ 
		board indicesDo: [ :aRow :aCol | 
			cell := board at: aRow at: aCol.
			html text: (cell printString);  text: '|'.
			aCol = board size second ifTrue: [ html break.] 
		].
		(1 to: board size second) do: [ :aCol | (board canMoveAt: aCol) 
			ifTrue: [ html anchor callback: [system player: system currentPlayer moveAt: aCol ]; with: ' $ ' ]
			ifFalse: [ html text: '  !!  ' ].
			html text: '|'.
			aCol = board size second ifTrue: [ html break.] 
		 ]
	].
	
	system currentGameType = #ultimate
	ifTrue: [ subboard := board at: 1 at: 1.
				(1 to: board size) do: [ :aRow | (1 to: subboard size) do: [ :sRow |
						(1 to: board size) do: [ :aCol |	
							(1 to: subboard size) do: [ :sCol |
								subboard := board at: aRow at: aCol.
								cell := subboard at: sRow at: sCol.
								((board canMoveAt: {aRow . aCol. sRow . sCol}) & ((board winner) = #undecided))
								ifTrue:	[ html anchor 
								callback: [	system player: system currentPlayer moveAt: {aRow . aCol . sRow . sCol} ];
								with: (cell printString) ]
								ifFalse: [ html text: (cell printString) ].
							sCol = (subboard size) ifTrue: [ html text: ' || '. 
								aCol = (board size) ifTrue: [ html break. sRow = (subboard size) ifTrue: [ html break ] ] ]
							ifFalse: [ html text: '|'  ] 
	] ] ] ] ].

	system currentGameType = #double_ultimate
	ifTrue: [ subboard := board at: 1 at: 1. ssubboard := subboard at: 1 at: 1.
				(1 to: board size) do: [ :aRow | (1 to: subboard size) do: [ :sRow | (1 to: ssubboard size) do: [ :ssRow |
					(1 to: board size) do: [ :aCol | (1 to: subboard size) do: [ :sCol | (1 to: ssubboard size) do: [ :ssCol |
						subboard := board at: aRow at: aCol. ssubboard := subboard at: sRow at: sCol.
						cell := ssubboard at: ssRow at: ssCol.
						((board canMoveAt: {aRow . aCol. sRow. sCol. ssRow. ssCol} ) and: ((board winner) = #undecided))
								ifTrue:	[ html anchor 
											   callback: [ system player: system currentPlayer moveAt: {aRow . aCol. sRow. sCol. ssRow. ssCol} ];
											   with: (cell printString) ]
								ifFalse: [ html text: (cell printString) ].
							(ssCol = (ssubboard size)) 
							ifTrue: [ html text: ' ||| ' ] 
							ifFalse: [ html text: '|' ].
							(aCol = (board size)  & sCol = (subboard size)  & ssCol = (ssubboard size)) 
							ifTrue: [ html break.  ssRow = (ssubboard size) ifTrue: [ html break ]. ]
		 ] ] ] ] ] ] ]! !

!UT3GameSession methodsFor: 'initialization' stamp: 'Oliver 7/25/2013 16:26'!
renderContentOn: html
	( ((system currentGameType) = #basic) | ((system currentGameType) = #medium) | ((system currentGameType) = #large) | ((system currentGameType) = #huge) | ((system currentGameType) = #gigantic) | ((system currentGameType) = #enormous) | ((system currentGameType) = #gomoku) | ((system currentGameType) = #ultimate) | ((system currentGameType) = #double_ultimate) | ((system currentGameType) = #connect4))
	ifTrue: [ 
		self renderSystemOn: html; 
		renderBoardOn: html.  
		]
	ifFalse: [
	system printString do: [ :char | 
		char = Character cr
		ifTrue: [ html break ]
		ifFalse: [ html text: char asString ] ].
	Transcript show: system printString.
	
	html break.
	html text: 'Move for Player '; text: system currentPlayer printString; break.
	html form: [ 
		html text: 'Row: '. html textInput on: #row of: self.
		html text: '  Column: '. html textInput on: #column of: self.
		system currentGameType = #ultimate
		ifTrue: [ 
			html text: '  Subrow: '. html textInput on: #subrow of: self.
			html text: '  Subcolumn: '. html textInput on: #subcolumn of: self.
		]
		ifFalse:[
			system currentGameType = #double_ultimate
			ifTrue: [ 
				html text: '  Subrow: '. html textInput on: #subrow of: self.
				html text: '  Subcolumn: '. html textInput on: #subcolumn of: self.
				html text: '  SubSubrow: '. html textInput on: #ssubrow of: self.
				html text: '  SubSubcolumn: '. html textInput on: #ssubcolumn of: self.
			]	
		].
		html submitButton text: 'Move'; callback: [ self makeMove ].
	].
	].
	html break.
	self renderGameSelectionOn: html.! !

!UT3GameSession methodsFor: 'initialization' stamp: 'Oliver 7/25/2013 16:03'!
renderGameSelectionOn: html
	| aiRadioGroup |
	html text: 'Reset game session: '; break.
	html form: [ 
		html select list: system gameTypes;
			on: #currentGameType of: system.
		html text: 'RandomAI: '.
		aiRadioGroup := html radioGroup.
		aiRadioGroup radioButton
			selected: withRandomAi;
			callback: [ withRandomAi := true. withEasyAi := false. withMediumAi := false. withHardAi := false. withInsaneAi := false.  withPerfectAi := false. withOutAi := false.
					     system resetWithGameType: ( system currentGameType ) ai: #random ].
		
		html text: 'Easy: '.
		aiRadioGroup radioButton
			selected: withEasyAi;
			callback: [ withRandomAi := false. withEasyAi := true. withMediumAi := false. withHardAi := false. withInsaneAi := false.  withPerfectAi := false. withOutAi := false.
					     system resetWithGameType: ( system currentGameType ) ai: #easy ].
		
		html text: 'Medium: '.
		aiRadioGroup radioButton
			selected: withMediumAi;
			callback: [ withRandomAi := false. withEasyAi := false. withMediumAi := true. withHardAi := false. withInsaneAi := false.  withPerfectAi := false. withOutAi := false. 
				           system resetWithGameType: ( system currentGameType ) ai: #med ].
			
		html text: 'Hard: '.
		aiRadioGroup radioButton
			selected: withHardAi;
			callback: [ withRandomAi := false. withEasyAi := false. withMediumAi := false. withHardAi := true. withInsaneAi := false.  withPerfectAi := false. withOutAi := false. 
				           system resetWithGameType: ( system currentGameType ) ai: #hard ].
			
		html text: 'InsaneWeighting: '.
		aiRadioGroup radioButton
			selected: withInsaneAi;
			callback: [ withRandomAi := false. withEasyAi := false. withMediumAi := false. withHardAi := false. withInsaneAi := true.  withPerfectAi := false. withOutAi := false. 
				           system resetWithGameType: ( system currentGameType ) ai: #insane ].
			
		html text: 'Perfect '.
		aiRadioGroup radioButton
			selected: withPerfectAi;
			callback: [ withRandomAi := false. withEasyAi := false. withMediumAi := false. withHardAi := false. withInsaneAi := false. withPerfectAi := true. withOutAi := false. 
				           system resetWithGameType: ( system currentGameType ) ai: #insane ].
			
		html text: 'NoAI: '.
		aiRadioGroup radioButton 
			selected: withOutAi;
			callback: [ withRandomAi := false. withEasyAi := false. withMediumAi := false. withHardAi := false. withInsaneAi := false. withPerfectAi := false. withOutAi := true. 
					     system resetWithGameType: (system currentGameType) ].
		html submitButton text: 'Go' 
		]! !

!UT3GameSession methodsFor: 'initialization' stamp: 'Oliver 7/7/2013 05:48'!
renderSystemOn: html
	html text: 'Current Game Type: '.
	html text: (system currentGameType printString).
	html break; text: 'Current Player: '.
	html text: (system currentPlayer printString).
	html break; text: 'Current Winning state: '.
	html text: (system board winner printString).
	html break.! !

!UT3GameSession methodsFor: 'initialization' stamp: 'ShineLow 7/4/2013 20:09'!
reset: aString 
	system resetWithGameType: aString asSymbol ! !

!UT3GameSession methodsFor: 'accessing' stamp: 'ShineLow 7/4/2013 20:38'!
row
	^ row asString! !

!UT3GameSession methodsFor: 'accessing' stamp: 'ShineLow 7/4/2013 20:39'!
row: aRow
	row := aRow asInteger! !

!UT3GameSession methodsFor: 'accessing' stamp: 'Oliver 7/5/2013 21:26'!
ssubcolumn
	^ ssubcolumn asString! !

!UT3GameSession methodsFor: 'accessing' stamp: 'Oliver 7/5/2013 21:26'!
ssubcolumn: aSubrow
	ssubcolumn := aSubrow asInteger! !

!UT3GameSession methodsFor: 'accessing' stamp: 'Oliver 7/5/2013 21:25'!
ssubrow
	^ ssubrow asString! !

!UT3GameSession methodsFor: 'accessing' stamp: 'Oliver 7/5/2013 21:25'!
ssubrow: aSubrow
	ssubrow := aSubrow asInteger! !

!UT3GameSession methodsFor: 'accessing' stamp: 'ShineLow 7/4/2013 20:38'!
subcolumn
	^ subcolumn asString! !

!UT3GameSession methodsFor: 'accessing' stamp: 'ShineLow 7/4/2013 20:38'!
subcolumn: aSubcolumn
	subcolumn := aSubcolumn asInteger! !

!UT3GameSession methodsFor: 'accessing' stamp: 'ShineLow 7/4/2013 20:38'!
subrow
	^ subrow asString! !

!UT3GameSession methodsFor: 'accessing' stamp: 'ShineLow 7/4/2013 20:38'!
subrow: aSubrow
	subrow := aSubrow asInteger! !

UT3WebServer subclass: #UT3MultiplayerHandler
	instanceVariableNames: 'sessions lock'
	classVariableNames: 'DefaultPort'
	poolDictionaries: ''
	category: 'CS528-UT3Server'!

!UT3MultiplayerHandler commentStamp: '<historical>' prior: 0!
An UT3MultiplayerHandler is a handler for a multiplayer session of the game. Note that there should only be one instance of this in the entire server.

Instance Variables
!

!UT3MultiplayerHandler class methodsFor: 'as yet unclassified' stamp: 'ShineLow 7/20/2013 07:05'!
canBeRoot 
	^ false! !

!UT3MultiplayerHandler class methodsFor: 'as yet unclassified' stamp: 'ShineLow 7/20/2013 11:12'!
clientHtml
	^ '<!!DOCTYPE html>
<meta charset="utf-8" />
<title>TicTacToe Basic</title>

<script language="javascript" type="text/javascript">

  var wsUri = "ws://" + location.host + "/tictactoe-multiplayer-servlet";
  var websocket;
  var output, handleInput, messageInput, sendButton, closeButton;

  function init() {
    output = document.getElementById("output");
    handleInput = document.getElementById("handleInput");
    messageInput = document.getElementById("messageInput");
    sendButton = document.getElementById("sendButton");
    closeButton = document.getElementById("closeButton");
    handleInput.value = "user-" + (new Date()).getMilliseconds();
    messageInput.onkeyup = function (evt) { if (evt.keyCode == 13) sendChatMessage(evt) }
    sendButton.onclick = function (evt) { sendChatMessage(evt) };
    closeButton.onclick = function (evt) { closeChat(evt) };
    testWebSocket();
  }

  function testWebSocket() {
    writeToScreen(wsUri);
    websocket = new WebSocket(wsUri);
    websocket.onopen = function(evt) { onOpen(evt) };
    websocket.onclose = function(evt) { onClose(evt) };
    websocket.onmessage = function(evt) { onMessage(evt) };
    websocket.onerror = function(evt) { onError(evt) };
  }

  function onOpen(evt) {
    writeToScreen("CONNECTED");
    /*doSend("Hi there !!");*/
  }

  function onClose(evt) {
    writeToScreen("DISCONNECTED");
  }

  function onMessage(evt) {
    writeToScreen(evt.data);
   }

  function onError(evt) {
    writeToScreen("<span style=''color: red''>ERROR:</span> " + evt.data);
  }

  function doSend(message) {
     websocket.send(message);
  }

  function writeToScreen(message) {
    var pre = document.createElement("p");
    pre.style.wordWrap = "break-word";
    pre.innerHTML = message;
    output.appendChild(pre);
  }

  function sendChatMessage(evt) {
    doSend(messageInput.value);
    messageInput.value = "";
  }

  function closeChat(evt) {
    doSend("Bye!!");
    websocket.close();
  }

  window.addEventListener("load", init, false);

</script>

<h2>Basic Tic-Tac-Toe</h2>
<div>
Handle: <input id="handleInput" type="text"/>
Move (eg: "1,1"): <input id="messageInput" type="text" size="64"/> 
<input id="sendButton" type="submit" value="Send"/>
<input id="closeButton" type="submit" value="Close"/>
</div>
<div id="output"></div>

</html> '
! !

!UT3MultiplayerHandler class methodsFor: 'as yet unclassified' stamp: 'ShineLow 7/20/2013 11:19'!
startUpOn: aPort
	"Start up the server with default options"
	|handlerString|
	handlerString := 'tictactoe-multiplayer'.
	ZnServer startDefaultOn: aPort.
	ZnServer default logToTranscript.
	ZnServer default delegate
		map: handlerString
  		to: [ :request | ZnResponse ok: (ZnEntity html: self clientHtml) ];
 		map: handlerString, '-servlet'
  		to: (ZnWebSocketDelegate map: handlerString, '-servlet' to: self new)! !

!UT3MultiplayerHandler methodsFor: 'as yet unclassified' stamp: 'ShineLow 7/20/2013 09:40'!
initialize
	sessions := OrderedCollection new.
	lock := Mutex new! !

!UT3MultiplayerHandler methodsFor: 'as yet unclassified' stamp: 'ShineLow 7/20/2013 10:04'!
register: aWebSocket 
	|available retval|
	lock critical: [ 
		available := (sessions select: [ :system | system isFull not ]) asSet.
		available size = 0
		ifTrue: [ 
			retval := UT3MultiplayerSystem new.
			sessions add: retval ]
		ifFalse: [ retval := available atRandom ].
		retval addPlayer: aWebSocket.
		].
	^ retval! !

!UT3MultiplayerHandler methodsFor: 'as yet unclassified' stamp: 'ShineLow 7/20/2013 09:52'!
value: aWebSocket 
	|system|
	system := self register: aWebSocket.
	system handlePlayer: aWebSocket.! !

UT3WebServer subclass: #UT3WebBase
	instanceVariableNames: 'session'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-UT3Server'!

!UT3WebBase commentStamp: '<historical>' prior: 0!
An UT3WebBase is base page containing all the web components of the game.

Instance Variables
	modeSelector:		<Object>

modeSelector
	- xxxxx
!

!UT3WebBase class methodsFor: 'testing' stamp: 'ShineLow 7/4/2013 19:54'!
canBeRoot
	^ true! !

!UT3WebBase class methodsFor: 'class initialization' stamp: 'ShineLow 7/4/2013 19:54'!
initialize
	super initialize.
	self registerAsApplication: 'Tic Tac Toe app'! !

!UT3WebBase methodsFor: 'hooks' stamp: 'ShineLow 7/4/2013 19:44'!
children
	^ { session }! !

!UT3WebBase methodsFor: 'initialization' stamp: 'ShineLow 7/4/2013 19:44'!
initialize 	
	super initialize.
	session := UT3GameSession new.! !

!UT3WebBase methodsFor: 'rendering' stamp: 'ShineLow 7/4/2013 19:43'!
renderContentOn: html
	html heading: 'Welcome to Ultimate Tic-Tac-Toe!!'.
	self children do: [ :each | html div class: 'subcomponent'; with: each ]! !

!UT3WebServer class methodsFor: 'as yet unclassified' stamp: 'ShineLow 7/20/2013 10:10'!
startUpOn: aPort1 multiplayer: aPort2 
	ZnZincServerAdaptor startOn: aPort1.
	UT3MultiplayerHandler startUpOn: aPort2! !

!UT3WebServer methodsFor: 'hooks' stamp: 'Oliver 7/7/2013 05:53'!
style
	^ 'body {
	font: 12pt Arial, Helvetica, sans-serif, Times New Roman;
}
h2 {
	font-size: 12pt;
	font-weight: normal;
	font-style: italic;
}
table {
	border-collapse: collapse;
}
td {
	border: 2px solid #CCCCCC;
	padding: 4px;
}
#author {
	border: 1px solid black;
	padding: 2px;
	margin: 2px;
}
.subcomponent {
	border: 2px solid lightblue;
	padding: 2px;
	margin: 2px;
}
.highlight {
	background-color: yellow;
}
.boolean {
	background-color: lightgrey;
}
.field {
	background-color: lightgrey;
}'! !

TestCase subclass: #UT3MultiplayerSystemTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-UT3Server'!

!UT3MultiplayerSystemTest methodsFor: 'as yet unclassified' stamp: 'ShineLow 7/20/2013 10:34'!
testDisconnectedPlayerCanReconnect
	|system mock1 mock2|
	system := UT3MultiplayerSystem new.
	self assert: ( system isFull not ).
	mock1 := UT3MockWebSocket new.
	mock2 := UT3MockWebSocket new.
	system	addAndHandlePlayer: mock1.
	system addAndHandlePlayer: mock2.
	self assert: ( system isFull ).
	self should: [ system addPlayer: mock1 ] raise: WebError.
	mock1 kill.
	"We need to wait here, because termination of the connection is not immediate."
	self assert: ( system isFull not ).
	mock1 := UT3MockWebSocket new.
	system addAndHandlePlayer: mock1.
	self assert: ( system isFull ).! !

!UT3MultiplayerSystemTest methodsFor: 'as yet unclassified' stamp: 'ShineLow 7/20/2013 09:19'!
testErrorWhenMovingInSameTurn
	|system mock1 mock2|
	system := UT3MultiplayerSystem new.
	self assert: ( system isFull not ).
	mock1 := UT3MockWebSocket new.
	mock2 := UT3MockWebSocket new.
	system	addAndHandlePlayer: mock1.
	system addAndHandlePlayer: mock2.
	self assert: ( system isFull ).
	mock1 receiveMessage: 'move:1,1'.
	self assert: ( mock1 receiveMessageAndExpect: 'move:1,1' ) equals: 'ERROR: Not your turn!!'.
	mock1 kill. mock2 kill.
	! !

!UT3MultiplayerSystemTest methodsFor: 'as yet unclassified' stamp: 'ShineLow 7/20/2013 11:06'!
testErrorWhenSendingUnknownFormat
	|system mock1 mock2 expected|
	system := UT3MultiplayerSystem new.
	self assert: ( system isFull not ).
	mock1 := UT3MockWebSocket new.
	mock2 := UT3MockWebSocket new.
	system	addAndHandlePlayer: mock1.
	system addAndHandlePlayer: mock2.
	self assert: ( system isFull ).
	expected := 'ERROR: Unknown header gibberish'.
	self assert: ( mock1 receiveMessageAndExpect: 'gibberish' ) equals: expected.
	self assert: ( mock2 receiveMessageAndExpect: 'gibberish:something' ) equals: expected.
	mock1 kill. mock2 kill.
	! !

!UT3MultiplayerSystemTest methodsFor: 'as yet unclassified' stamp: 'ShineLow 7/20/2013 09:05'!
testInstantiateWithBothPlayers
	|system mock1 mock2|
	system := UT3MultiplayerSystem new.
	self assert: ( system isFull not ).
	mock1 := UT3MockWebSocket new.
	mock2 := UT3MockWebSocket new.
	system	addAndHandlePlayer: mock1.
	system addAndHandlePlayer: mock2.
	self assert: ( system isFull ).
	self should: [ system addPlayer: mock1 ] raise: WebError.
	mock1 kill.
	mock2 kill.
	"We need to wait here, because termination of the connection is not immediate."
	self assert: ( system isFull not )! !

!UT3MultiplayerSystemTest methodsFor: 'as yet unclassified' stamp: 'ShineLow 7/20/2013 07:50'!
testParseCoords
	|system|
	system := UT3MultiplayerSystem new.
	self assert: ( system parseCoords: '1,2,3' ) equals: { 1. 2. 3 }.
	self assert: ( system parseCoords: 'gibberish' ) equals: nil.
	self assert: ( system parseCoords: '1,2,monkeys' ) equals: nil! !

!UT3MultiplayerSystemTest methodsFor: 'as yet unclassified' stamp: 'ShineLow 7/20/2013 10:56'!
testPlayOneBasicGame
	|system mock1 mock2|
	system := UT3MultiplayerSystem new.
	self assert: ( system isFull not ).
	mock1 := UT3MockWebSocket new.
	mock2 := UT3MockWebSocket new.
	system	addAndHandlePlayer: mock1.
	system addAndHandlePlayer: mock2.
	self assert: ( system isFull ).
	mock1 receiveMessage: 'move:1,1'.
	mock2 receiveMessage: 'move:1,2'.
	mock1 receiveMessage: 'move:2,2'.
	mock2 receiveMessage: 'move:1,3'.
	mock1 receiveMessage: 'move:3,3'.
	self assert: ( system isGameOver ).
	mock1 kill. mock2 kill.
	! !

!UT3MultiplayerSystemTest methodsFor: 'as yet unclassified' stamp: 'ShineLow 7/20/2013 10:57'!
testPlayOneUltimateGame
	|system mock1 mock2|
	system := UT3MultiplayerSystem new.
	self assert: ( system isFull not ).
	mock1 := UT3MockWebSocket new.
	mock2 := UT3MockWebSocket new.
	system	addAndHandlePlayer: mock1.
	system addAndHandlePlayer: mock2.
	self assert: ( system isFull ).
	mock1 receiveMessage: 'reset:ultimate'.
	mock1 receiveMessage: 'move:2,2,2,2'.
	mock2 receiveMessage: 'move:2,2,1,2'.
	mock1 receiveMessage: 'move:1,2,1,3'.
	mock2 receiveMessage: 'move:1,3,1,2'.
	mock1 receiveMessage: 'move:1,2,2,2'.
	mock2 receiveMessage: 'move:2,2,1,1'.
	mock1 receiveMessage: 'move:1,1,2,2'.
	mock2 receiveMessage: 'move:2,2,1,3'.
	self assert: ( system isGameOver not).
	mock1 receiveMessage: 'move:1,3,2,3'.
	mock2 receiveMessage: 'move:2,3,2,2'.
	mock1 receiveMessage: 'move:2,2,2,3'.
	mock2 receiveMessage: 'move:2,3,1,3'.
	mock1 receiveMessage: 'move:1,3,1,1'.
	mock2 receiveMessage: 'move:1,1,3,3'.
	mock1 receiveMessage: 'move:3,3,2,3'.
	mock2 receiveMessage: 'move:2,3,3,1'.
	self assert: ( system isGameOver not).
	mock1 receiveMessage: 'move:3,1,2,1'.
	mock2 receiveMessage: 'move:2,1,2,2'.
	mock1 receiveMessage: 'move:2,2,2,1'.
	mock2 receiveMessage: 'move:2,1,2,3'.
	mock1 receiveMessage: 'move:2,3,2,1'.
	mock2 receiveMessage: 'move:2,1,2,1'.
	self assert: ( system isGameOver ).
	mock1 kill. mock2 kill.
	! !

!UT3MultiplayerSystemTest methodsFor: 'as yet unclassified' stamp: 'ShineLow 7/20/2013 09:04'!
testResetBoards
	|system mock1 mock2|
	system := UT3MultiplayerSystem new.
	self assert: ( system isFull not ).
	mock1 := UT3MockWebSocket new.
	mock2 := UT3MockWebSocket new.
	system	addAndHandlePlayer: mock1.
	system addAndHandlePlayer: mock2.
	self assert: ( system isFull ).
	mock1 receiveMessage:  'reset:ultimate'.
	self assert: ( system currentGameType) equals: #ultimate.
	mock1 receiveMessage: 'reset:basic'.
	self assert: ( system currentGameType ) equals: #basic.
! !

UT3System subclass: #UT3MultiplayerSystem
	instanceVariableNames: 'lock sendLock system p1Socket p2Socket headers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-UT3Server'!

!UT3MultiplayerSystem commentStamp: '<historical>' prior: 0!
An UT3MultiplayerSystem is a wrapper around one session of web multiplayer, between two players. Contains handling of messages, turns, etc.

Instance Variables
	p1Socket:		<Object>
	p2Socket:		<Object>
	system:		<Object>

p1Socket
	- xxxxx

p2Socket
	- xxxxx

system
	- xxxxx
!

!UT3MultiplayerSystem methodsFor: 'public' stamp: 'ShineLow 7/20/2013 06:27'!
addAndHandlePlayer: aWebSocket
	"Convenience method, that adds a player, and also handles it, by forking a separate process to deal with it.
	This method does terminate, but you have to be aware that the handling is still running in the background."
	self addPlayer: aWebSocket.
	[ self handlePlayer: aWebSocket  ] fork.! !

!UT3MultiplayerSystem methodsFor: 'public' stamp: 'ShineLow 7/20/2013 10:03'!
addPlayer: aWebSocket 
	"Add the player"
	|thisPlayer|
	thisPlayer := nil.
	lock critical: [ 
		p1Socket isNil
		ifTrue: [ p1Socket := aWebSocket. thisPlayer := 1 ]
		ifFalse: [ p2Socket isNil 
				  ifTrue: [ p2Socket := aWebSocket. thisPlayer := 2 ] ].
		thisPlayer isNil ifFalse: [ system resetWithGameType: system currentGameType ] ].
	thisPlayer isNil ifTrue: [ WebError signal: 'Cannot add to a full game' ]
	! !

!UT3MultiplayerSystem methodsFor: 'sending' stamp: 'ShineLow 7/20/2013 11:03'!
broadcast: aString
	sendLock critical: [ 
	[ p1Socket isNil ifFalse: [ p1Socket sendMessage: aString ] ]
		on: NetworkError do: [ :exception | Transcript show: 'Error broadcasting to p1';cr. ].
	[ p2Socket isNil ifFalse: [ p2Socket sendMessage: aString ] ]
		on: NetworkError do: [ :exception | Transcript show: 'Error broadcasting to p2';cr. ]
	]! !

!UT3MultiplayerSystem methodsFor: 'sending' stamp: 'ShineLow 7/20/2013 11:04'!
broadcastState
	p1Socket isNil ifFalse: [ self sendSystemStateTo: p1Socket ].
	p2Socket isNil ifFalse: [ self sendSystemStateTo: p2Socket ]! !

!UT3MultiplayerSystem methodsFor: 'accessing' stamp: 'ShineLow 7/20/2013 08:00'!
currentGameType 
	^ system currentGameType ! !

!UT3MultiplayerSystem methodsFor: 'public' stamp: 'ShineLow 7/20/2013 10:03'!
handlePlayer: aWebSocket 
	"Handle player is a blocking function, that handles the messages from the given webSocket.
	It terminates and returns only when the connection has died. You should have already call addPlayer"
	|thisPlayer|
	lock critical: [ 
		(( aWebSocket == p1Socket ) not & ( aWebSocket == p2Socket ) not)
		ifTrue: [ WebError signal: 'Socket not added to system' ]
		 ].
	[ aWebSocket == p1Socket ifTrue: [ thisPlayer := 1 ] ifFalse: [ thisPlayer := 2 ].
	  self player: thisPlayer sendMessage: 'Welcome!! You are now player ',thisPlayer asString.
	  self broadcast: 'Player ',thisPlayer asString,' has joined the game'.
	  self broadcast: 'Current turn is player ',system currentPlayer asString.
	  aWebSocket runWith: [ :message | self player: thisPlayer handle: message ] ]
		on: ConnectionClosed
		do: [ self removePlayer: aWebSocket ]
							! !

!UT3MultiplayerSystem methodsFor: 'public' stamp: 'ShineLow 7/20/2013 11:03'!
initialize
	lock := Mutex new.
	sendLock := Mutex new.
	system := UT3System new.
	system resetWithGameType: #basic.
	p1Socket := nil.
	p2Socket := nil.
	
	headers := Set newFrom: { 'reset'. 'move'. 'show' }! !

!UT3MultiplayerSystem methodsFor: 'public' stamp: 'ShineLow 7/20/2013 06:07'!
isFull
	lock critical: [ p1Socket isNil ifTrue: [ ^ false]. p2Socket isNil ifTrue: [ ^ false ]. ^true ]! !

!UT3MultiplayerSystem methodsFor: 'accessing' stamp: 'ShineLow 7/20/2013 08:01'!
isGameOver 
	^ system isGameOver ! !

!UT3MultiplayerSystem methodsFor: 'handling' stamp: 'ShineLow 7/20/2013 07:48'!
parseCoords: coordStringList 
	"Returns nil if failed to parse"
	|coords|
	coords := ( coordStringList findTokens: ',' ) collect: [ :char | char asInteger ].
	(coords includes: nil) ifTrue: [ ^nil ] ifFalse: [ ^coords asArray ]! !

!UT3MultiplayerSystem methodsFor: 'handling' stamp: 'ShineLow 7/20/2013 11:21'!
player: aNumber handle: message
	"Takes a message, parse it, and then sends the response back, possibly broadcasting to everyone.
	Note that the sending of the message itself should not be in the lock, as we don't want to block another player B if sending
	sending back to player A is slow over the network.
	
	message format must of the form: '<header>:<value>'
	header must be one defined under the instance value headers.
	value is dependent on the header. Eg. if header value is reset, then value should be a valid game type.
	if header value is move, then value should be a comma separated list of coordinates
	"
	|errMsg tokens header coords |
	Transcript show:'From: ';show:aNumber asString;cr;show:message;cr.
	errMsg := nil.
	tokens := message findTokens: ':'.
	header := tokens first.
	(headers includes: header) ifFalse: [ ^ self player: aNumber sendError: 'Unknown header ',header ].
	lock critical: [ 
		system currentPlayer = aNumber
		ifTrue: [ header = 'reset'
				 ifTrue: [ 
					(tokens size = 2)
					ifTrue: [ errMsg := self player: aNumber reset: tokens second asSymbol ]
					ifFalse: [ errMsg := 'Invalid message format' ] ].
				 header = 'move'
				 ifTrue: [
					(tokens size = 2)
					ifTrue: [ 
						coords := self parseCoords: tokens second.
						coords isNil ifTrue: [ errMsg := 'Invalid coordinate format' ]
									 ifFalse: [ errMsg := self player: aNumber move: coords ] ] ]
				]
		ifFalse: [ errMsg := 'Not your turn!!' ] ].
	errMsg isNil ifTrue: [ ^ self broadcastState]
				 ifFalse: [ ^ self player: aNumber sendError: errMsg ]
							! !

!UT3MultiplayerSystem methodsFor: 'handling' stamp: 'ShineLow 7/20/2013 08:47'!
player: aNumber move: coordArray
	[ 
	  (( coordArray size = 2 ) | (coordArray size = 4)) ifFalse: [ ^'Invalid size of coordinates array' ].
	  coordArray size = 2
	  ifTrue: [ system player: aNumber moveRow: coordArray first column: coordArray second ] .
	  coordArray size = 4
	  ifTrue: [ system player: aNumber moveAt: { coordArray first. coordArray second }
			   subBoard: { coordArray third. coordArray fourth } ].
	] on: SystemError do: [ :exception | ^'Invalid move: ', exception printString ].
	^nil
	! !

!UT3MultiplayerSystem methodsFor: 'handling' stamp: 'ShineLow 7/20/2013 07:55'!
player: aNumber reset: aGameType
	"Returns nil if OK"
	(system gameTypes includes: aGameType) ifFalse: [ ^ 'Unknown game type', aGameType asString ].
	system resetWithGameType: aGameType.
	^ nil! !

!UT3MultiplayerSystem methodsFor: 'sending' stamp: 'ShineLow 7/20/2013 08:16'!
player: aNumber sendError: aString
	self player: aNumber sendMessage: 'ERROR: ',aString! !

!UT3MultiplayerSystem methodsFor: 'sending' stamp: 'ShineLow 7/20/2013 11:04'!
player: aNumber sendMessage: aString
	|socket|
	sendLock critical: [ 
	aNumber = 1 ifTrue: [ socket := p1Socket  ] ifFalse: [ socket := p2Socket  ].
	[ socket sendMessage: aString ]
		on: NetworkError
		do: [ :exception | Transcript show: 'Network error sending error message';cr.
						   self removePlayer: socket ].
	]! !

!UT3MultiplayerSystem methodsFor: 'printing' stamp: 'ShineLow 7/20/2013 06:02'!
printOn: aStream
	system printOn: aStream.
	p1Socket printOn: aStream.
	p2Socket printOn: aStream ! !

!UT3MultiplayerSystem methodsFor: 'public' stamp: 'ShineLow 7/20/2013 08:09'!
removePlayer: aWebSocket 
	lock critical: [ 
		p1Socket == aWebSocket ifTrue: [ p1Socket := nil ].
		p2Socket == aWebSocket ifTrue: [ p2Socket := nil ].
		system resetWithGameType: system currentGameType ].
	self broadcast: 'The other player has been disconnected. Please wait while a new player joins.'
							! !

!UT3MultiplayerSystem methodsFor: 'sending' stamp: 'ShineLow 7/20/2013 11:04'!
sendSystemStateTo: aWebSocket
	| rowStrings |
	sendLock critical: [
	[ aWebSocket sendMessage: 'Current game type: ', system currentGameType asString;
				     sendMessage: 'Current player: ', system currentPlayer asString;
				     sendMessage: 'Current winner: ', system board winner asString.
	  rowStrings := system board getRowStrings.
	  rowStrings do: [ : rowString | aWebSocket sendMessage: rowString ].
	"
	  system currentGameType = #basic ifTrue: [
		1 to: system board size do: [ 
			: row |
			rowString := WriteStream on: ( String new ).
			system board printRow: row on: rowString.
			aWebSocket sendMessage: rowString contents ] ]."
	 ] on: NetworkError do: [ 
		:exception | Transcript show: 'Network error sending error message';cr.
					 self removePlayer: aWebSocket  ]
	]! !

Error subclass: #WebError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-UT3Server'!

!WebError commentStamp: '<historical>' prior: 0!
A WebError is an error signalling a web error.

Instance Variables
!
UT3WebBase initialize!
